@@ -2,6 +2,7 @@
 from plone.app.upgrade.utils import alias_module
 
 from . import bbb
+from . import bbb_qi
 from . import bbbd
 import pkg_resources
 import sys
@@ -178,6 +179,12 @@ except ImportError:
     from OFS.LockItem import LockItem
     alias_module('webdav.LockItem.LockItem', LockItem)
 
+try:
+    pkg_resources.get_distribution('Products.CMFQuickInstallerTool')
+except pkg_resources.DistributionNotFound:
+    alias_module('Products.CMFQuickInstallerTool', bbb_qi)
+    alias_module('Products.CMFPlone.QuickInstallerTool', bbb_qi)
+
 
 class HiddenProducts(object):
     """This hides the upgrade profiles from the quick installer tool."""
